/*
 * GradientLabel is part of the RADi GUI Builder package.
 * All rights reserved by muntjak Software (www.muntjak.com).
 */
 
package de.muntjak.customcomponents;

import java.awt.*;

import javax.swing.*;
import javax.swing.border.*;

/**
 * GradientLabel is an opaque label with a gradient background and a line border.
 * The label is painted antialiased.
 * Default colors should work well with Metal 1.5.
 * 
 * @version 1.0
 * @author Hans Bickel
 */
public class GradientLabel extends JLabel {

	// defaults
	private static final Insets defaultInsets = new Insets(1, 4, 2, 4);
	private static final Font defaultFont = new Font("Dialog", Font.BOLD, 12);
	private static final Color defaultForeground = new Color(64, 64, 64);
	private static final RenderingHints renderingHints =
		new RenderingHints(RenderingHints.KEY_ANTIALIASING,
		RenderingHints.VALUE_ANTIALIAS_ON);

	// public properties
	private Color borderColor = new Color(153, 173, 191);
	private Color startColor = new Color(190, 211, 231);
	private Color endColor = new Color(240, 245, 249);
	private Insets margin = defaultInsets;
	private int bias = 50;
	private boolean textAntialiased = true;
	
	// private flag, only true when in paintComponent(Graphics)
	private boolean inPaint = false;

	public GradientLabel() {
        this("GradientLabel", null, LEADING);
    }
    
    public GradientLabel(String text) {
        this(text, null, LEADING);
    }
    
    public GradientLabel(String text, int horizontalAlignment) {
        this(text, null, horizontalAlignment);
    }
    
    public GradientLabel(String text, Icon icon, int horizontalAlignment) {
        setText(text);
        setIcon(icon);
        setHorizontalAlignment(horizontalAlignment);
        updateUI();
        setAlignmentX(LEFT_ALIGNMENT);
        
        // set defaults
        super.setOpaque(true);
        super.setFont(defaultFont);
        super.setForeground(defaultForeground);
        super.setBorder(new GradientLabelBorder());
    }
    
    public GradientLabel(Icon image) {
        this(null, image, CENTER);
    }
    
    public GradientLabel(Icon image, int horizontalAlignment) {
        this(null, image, horizontalAlignment);
    }

    public boolean isOpaque() {
    	if(inPaint) return false;
    	
    	return super.isOpaque();
    }

    public void paintComponent(Graphics g) {
    	if(super.isOpaque()) {
	    	inPaint = true;
	    	
	    	if(g instanceof Graphics2D) {
				Graphics2D g2d = (Graphics2D)g;
				
				if(textAntialiased) {
					g2d.addRenderingHints(renderingHints);
				}
	
				// bias is between (0, 100) where 50 is centered (0, width),
				// 25 is (0, w / 2), 75 is (w / 2, w)
				int w = getWidth();
				int x1 = 0;
				if(bias > 50) {
					//increase x1
					x1 = w * (bias - 50) / 50;
				}
				
				int x2 = w;
				if(bias < 50) {
					// decrease x2
					x2 = w * bias / 50;
				}
				
				g2d.setPaint(new GradientPaint(
					x1, 0, startColor, x2, 0, endColor));
			}
			
			// draw the background
			g.fillRect(0, 0, getWidth(), getHeight());
    	}
		
		// draw the label
		super.paintComponent(g);
		
		inPaint = false;
    }
    
    class GradientLabelBorder implements Border {

		public boolean isBorderOpaque() {
			return false;
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int w, int h) {	
			g.setColor(borderColor);
			g.drawRect(x, y, w - 1, h - 1);
		}

		public Insets getBorderInsets(Component c) {
			return margin;
		}
    }

	public Color getBorderColor() {
		return borderColor;
	}

	public Color getStartColor() {
		return startColor;
	}

	public Color getEndColor() {
		return endColor;
	}

	public void setBorderColor(Color color) {
		borderColor = color;
		repaint();
	}

	public void setStartColor(Color color) {
		startColor = color;
		repaint();
	}

	public void setEndColor(Color color) {
		endColor = color;
		repaint();
	}

	public Insets getMargin() {
		return margin;
	}

	public void setMargin(Insets insets) {
		margin = insets;
		repaint();
	}

	public int getBias() {
		return bias;
	}

	public void setBias(int i) {
		bias = i;
		
		if(i < 1) bias = 1;
		else if(i > 100) bias = 100;
		
		repaint();
	}

	public boolean isTextAntialiased() {
		return textAntialiased;
	}

	public void setTextAntialiased(boolean b) {
		textAntialiased = b;
	}
}
